@@ -41,7 +41,7 @@
 #ifdef HP700
 extern  sc_r1tor18( );
 #endif
-#if defined(LINUX) || defined(FREEBSD) || defined(AMD64)
+#if defined(LINUX) || defined(FREEBSD) || defined(AMD64) || defined(OPENBSD)
 extern  sc_geti386regs( S2CINT* a );
 #endif
 #ifdef MC680X0
@@ -219,7 +219,7 @@ static  trace_stack_and_registers()
 #endif
 
 #if defined(AMD64) || defined(LINUX) || defined(HP700) \
-  || defined(MIPS) || defined(FREEBSD)
+ || defined(MIPS) || defined(FREEBSD) || defined(LINUX_ARM) || defined(OPENBSD)
 /* The following code is used to read the stack pointer.  The register
    number is passed in to force an argument to be on the stack, which in
    turn can be used to find the address of the top of stack.
@@ -262,7 +262,7 @@ static void trace_stack_and_registers()
 }
 #endif
 
-#ifdef LINUX
+#if defined(LINUX) || defined(OPENBSD)
 /* All processor registers which might contain pointers are traced by the
    following procedure.
 */
@@ -400,6 +400,20 @@ static void trace_stack_and_registers()
 }
 #endif
 
+#ifdef LINUX_ARM
+/* All processor registers which might contain pointers are traced by the
+   following procedure.
+*/
+
+static void trace_stack_and_registers()
+{
+      S2CINT  armregs[9], *pp;
+
+      sc_getARMregs( armregs );
+      STACKPTR( pp );
+      while  (pp != sc_stackbase)  move_continuation_ptr( ((SCP)*pp++) );
+}
+#endif
 
 /* The size of an extended object in words is returned by the following
    function.
@@ -1740,12 +1754,12 @@ failed:
 	if  (tag == PAIRTAG)  {
 	   sc_conscnt = PAGEBYTES/CONSBYTES;
 	   sc_consp = (SCP)PAGE_ADDRESS( page );
-	   if  (sc_collecting)  QUEUE_PAGE( cons_pages, page );
+	   if (sc_collecting) { QUEUE_PAGE( cons_pages, page ); }
 	}
 	else  {
 	   sc_extobjp = (SCP)PAGE_ADDRESS( page );
 	   sc_extobjwords = count*PAGEWORDS;
-	   if  (sc_collecting)  QUEUE_PAGE( extended_pages, page );
+	   if (sc_collecting) { QUEUE_PAGE( extended_pages, page ); }
 	   while (--count)  {
 	      sc_pagegeneration[ ++page ] = sc_next_generation;
 	      sc_pagetype[ page ] = BIGEXTENDEDTAG;
@@ -1931,7 +1945,9 @@ struct SEEN*  seenp;
 
 /* Put a breakpoint on this procedure to catch verification problems */
 
+#ifdef __GNUC__
 static void verifyfail() __attribute__((noreturn));
+#endif
 static void verifyfail() 
 {
 	sc_abort();
@@ -2028,11 +2044,11 @@ TSCP  sc_verifyobject( TSCP any )
 	      sc_verifyobject( PAIR_CDR( any ) );
 	      seenp = seen.prev;
 	      return( any );
- 	   default:
+	   default:
 	     verifyfail();
 	}
 }
-	      
+
 /* The following function forms a weak dotted-pair with any two Scheme
    pointers.  A weak dotted-pair is a pair that has the property that the CAR
    of the pair may be set to #F by the garbage collector if it contains the
@@ -2056,7 +2072,3 @@ TSCP  sc_weak_2dcons( TSCP x, TSCP y )
 	MUTEXOFF;
 	return( cons );
 }
-	
-
-		 
-                              
