@@ -811,6 +811,10 @@ extern TSCP  sc_scheme_2dmode();
 
 extern TSCP  sc_set_2dscheme_2dmode_21( XAL1( TSCP ) );
 
+extern TSCP  sc_osexit( XAL1( TSCP ) );
+
+extern void sc_segv__handlers();
+
 /* The definitions which follow are used by the code generated by the Scheme->C
    compiler.  They are included in this file so that only one #include file
    will be required.
