@@ -3,6 +3,7 @@
 
 #include <objects.h>
 
+void scexpnd2__init();
 DEFSTATICTSCP( c2751 );
 DEFSTATICTSCP( t2832 );
 DEFSTATICTSCP( t2833 );
@@ -935,6 +936,12 @@ TSCP  scexpnd2_l2748( x2749, e2750 )
                                                     PROCEDURE_CLOSURE( X1 ) ) );
 }
 
+void scdebug__init();
+void scrt2__init();
+void scrt6__init();
+void scrt1__init();
+void scexpand__init();
+
 static void  init_modules( compiler_version )
         char *compiler_version;
 {
@@ -956,6 +963,7 @@ void  scexpnd2__init()
         INITHEAP( 0, 0, 0, 0 );
         init_constants();
         init_modules( "(scexpnd2 SCHEME->C COMPILER 15mar93jfb)" );
+        sc_segv__handlers();
         INITIALIZEVAR( t2841, 
                        ADR( scexpnd2_let_2dmacro_v ), 
                        MAKEPROCEDURE( 1, 
