@@ -3,6 +3,7 @@
 
 #include <objects.h>
 
+void callcode__init();
 DEFSTATICTSCP( lambda_2dbody_2dgenc_v );
 DEFSTATICTSCP( report_2derror_v );
 DEFSTATICTSCP( reserve_2ddisplay_v );
@@ -1602,6 +1603,14 @@ L3858:
         POPSTACKTRACE( X2 );
 }
 
+void scdebug__init();
+void expform__init();
+void scrt2__init();
+void lap__init();
+void gencode__init();
+void plist__init();
+void scrt1__init();
+
 static void  init_modules( compiler_version )
         char *compiler_version;
 {
@@ -1623,6 +1632,7 @@ void  callcode__init()
         INITHEAP( 0, 0, 0, 0 );
         init_constants();
         init_modules( "(callcode SCHEME->C COMPILER 15mar93jfb)" );
+        sc_segv__handlers();
         INITIALIZEVAR( t3335, 
                        ADR( callcode__24call_2dgenc_v ), 
                        MAKEPROCEDURE( 3, 
